@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/DUX/osi_vfsops.c,v 1.1.1.5 2001/07/14 22:19:38 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/DUX/osi_vfsops.c,v 1.1.1.7 2002/05/10 23:43:40 hartmans Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -90,10 +90,10 @@ int mp_afs_mount(struct mount *afsp,char * path, caddr_t data,
     if ( !afsp->m_stat.f_mntonname || !afsp->m_stat.f_mntfromname)
 	panic("malloc failure in afs_mount\n");
 
-    bzero(afsp->m_stat.f_mntonname, MNAMELEN);
-    bzero(afsp->m_stat.f_mntfromname, MNAMELEN);
+    memset(afsp->m_stat.f_mntonname, 0, MNAMELEN);
+    memset(afsp->m_stat.f_mntfromname, 0, MNAMELEN);
     AFS_COPYINSTR(path, (caddr_t)afsp->m_stat.f_mntonname, MNAMELEN, &size, code);
-    bcopy("AFS", afsp->m_stat.f_mntfromname, 4);
+    memcpy(afsp->m_stat.f_mntfromname, "AFS", 4);
     AFS_GUNLOCK();
     (void) mp_afs_statfs(afsp);
     AFS_GLOCK();
@@ -148,15 +148,16 @@ int mp_afs_root (struct mount *afsp, struct vnode **avpp)
 	}
     }
     if (tvp) {
+	struct vnode *vp = AFSTOV(tvp);
 	AFS_GUNLOCK();
-	VN_HOLD((struct vnode *)tvp);
-	VN_LOCK((struct vnode *)tvp);
-	tvp->v.v_flag |= VROOT;	    /* No-op on Ultrix 2.2 */
-	VN_UNLOCK((struct vnode *)tvp);
+	VN_HOLD(vp);
+	VN_LOCK(vp);
+	vp->v_flag |= VROOT;	    /* No-op on Ultrix 2.2 */
+	VN_UNLOCK(vp);
 	AFS_GLOCK();
 
 	afs_globalVFS = afsp;
-	*avpp = (struct vnode *) tvp;
+	*avpp = vp;
     }
 
     afs_Trace2(afs_iclSetp, CM_TRACE_VFSROOT, ICL_TYPE_POINTER, *avpp,
@@ -273,7 +274,7 @@ int mp_afs_vptofh(struct vnode *avn, struct fid *fidp)
     long addr[2];
     register struct cell *tcell;
     int rootvp = 0;
-    struct vcache *avc = (struct vcache *)avn;
+    struct vcache *avc = VTOAFS(avn);
 
     AFS_GLOCK();
     AFS_STATCNT(afs_fid);
@@ -299,7 +300,7 @@ int mp_afs_vptofh(struct vnode *avn, struct fid *fidp)
 	fidp->fid_reserved = AFS_XLATOR_MAGIC;
 	addr[0] = (long)avc;
 	AFS_GUNLOCK();
-	VN_HOLD((struct vnode *)avc);
+	VN_HOLD(AFSTOV(avc));
 	AFS_GLOCK();
     }
 
@@ -307,12 +308,12 @@ int mp_afs_vptofh(struct vnode *avn, struct fid *fidp)
     fidp->fid_len = AFS_SIZEOFSMALLFID;
     if (afs_NFSRootOnly) {
 	if (rootvp) {
-	    bcopy((caddr_t)&Sfid, fidp->fid_data, AFS_FIDDATASIZE);   
+	    memcpy(fidp->fid_data, (caddr_t)&Sfid, AFS_FIDDATASIZE);   
 	} else {
-	    bcopy((caddr_t)addr, fidp->fid_data, AFS_FIDDATASIZE);   
+	    memcpy(fidp->fid_data, (caddr_t)addr, AFS_FIDDATASIZE);   
 	}
     } else {
-	bcopy((caddr_t)&Sfid, fidp->fid_data, AFS_FIDDATASIZE);   
+	memcpy(fidp->fid_data, (caddr_t)&Sfid, AFS_FIDDATASIZE);   
     }
     AFS_GUNLOCK();
     return 0;
