@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/IRIX/osi_vfsops.c,v 1.1.1.4 2001/07/14 22:19:43 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/IRIX/osi_vfsops.c,v 1.1.1.6 2002/05/10 23:43:59 hartmans Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -242,7 +242,7 @@ cred_t *cr;
      * rootvp gets lots of ref counts
      */
     if (rootvp) {
-	tvc = (struct vcache *)rootvp;
+	tvc = VTOAFS(rootvp);
         if (tvc->opens || CheckLock(&tvc->lock) || LockWaiters(&tvc->lock)) {
 	    ReleaseWriteLock(&afs_xvcache);
 	    return EBUSY;
@@ -290,13 +290,13 @@ afs_root (OSI_VFS_ARG(afsp), avpp)
     }
     if (tvp) {
 	int s;
-	VN_HOLD((struct vnode *)tvp);
-	s = VN_LOCK((struct vnode *)tvp);
-	tvp->v.v_flag |= VROOT;	
-	VN_UNLOCK((struct vnode *)tvp, s);
+	VN_HOLD(AFSTOV(tvp));
+	s = VN_LOCK(AFSTOV(tvp));
+	AFSTOV(tvp)->v_flag |= VROOT;	
+	VN_UNLOCK(AFSTOV(tvp), s);
 
 	afs_globalVFS = afsp;
-	*avpp = (struct vnode *) tvp;
+	*avpp = AFSTOV(tvp);
     }
 
     afs_Trace2(afs_iclSetp, CM_TRACE_VFSROOT, ICL_TYPE_POINTER, *avpp,
@@ -523,7 +523,7 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t **avcp, struct fid *fidp)
     register afs_int32 code = 0;
     afs_int32 ret;
 
-#if defined(AFS_SGI64_ENV) && defined(CKPT)
+#if defined(AFS_SGI64_ENV) && defined(CKPT) && !defined(_R5000_CVT_WAR)
     afs_fid2_t *afid2;
 #endif    
 
@@ -533,11 +533,11 @@ afs_vget(OSI_VFS_DECL(afsp), vnode_t **avcp, struct fid *fidp)
 
     *avcp = NULL;
 
-#if defined(AFS_SGI64_ENV) && defined(CKPT)
+#if defined(AFS_SGI64_ENV) && defined(CKPT) && !defined(_R5000_CVT_WAR)
     afid2 = (afs_fid2_t*)fidp;
     if (afid2->af_len == sizeof(afs_fid2_t) - sizeof(afid2->af_len)) {
 	/* It's a checkpoint restart fid. */
-	tcell = afs_GetCellByIndex(afid2->af_cell, READ_LOCK);
+	tcell = afs_GetCellByIndex(afid2->af_cell, READ_LOCK, 0 /* !refresh */);
 	if (!tcell) {
 	    code = ENOENT;
 	    goto out;
