@@ -23,11 +23,12 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.1.1.12 2001/09/20 06:07:27 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/LINUX/osi_vnodeops.c,v 1.1.1.15 2002/05/10 23:44:10 hartmans Exp $");
 
 #include "../afs/sysincludes.h"
 #include "../afs/afsincludes.h"
 #include "../afs/afs_stats.h"
+#include "../afs/afs_osidnlc.h"
 #include "../h/mm.h"
 #include "../h/pagemap.h"
 #if defined(AFS_LINUX24_ENV)
@@ -60,7 +61,7 @@ static ssize_t afs_linux_read(struct file *fp, char *buf, size_t count,
 			      loff_t *offp)
 {
     ssize_t code;
-    struct vcache *vcp = (struct vcache*)fp->f_dentry->d_inode;
+    struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
     cred_t *credp = crref();
     struct vrequest treq;
 
@@ -104,7 +105,7 @@ static ssize_t afs_linux_write(struct file *fp, const char *buf, size_t count,
 {
     ssize_t code = 0;
     int code2;
-    struct vcache *vcp = (struct vcache *)fp->f_dentry->d_inode;
+    struct vcache *vcp = ITOAFS(fp->f_dentry->d_inode);
     struct vrequest treq;
     cred_t *credp = crref();
 
@@ -157,7 +158,7 @@ static int afs_linux_readdir(struct file *fp,
 			     void *dirbuf, filldir_t filldir)
 {
     extern struct DirEntry * afs_dir_GetBlob();
-    struct vcache *avc = (struct vcache*)FILE_INODE(fp);
+    struct vcache *avc = ITOAFS(FILE_INODE(fp));
     struct vrequest treq;
     register struct dcache *tdc;
     int code;
@@ -314,7 +315,7 @@ void afs_linux_vma_close(struct vm_area_struct *vmap)
     if (!vmap->vm_file)
 	return;
 
-    vcp = (struct vcache*)FILE_INODE(vmap->vm_file);
+    vcp = ITOAFS(FILE_INODE(vmap->vm_file));
     if (!vcp)
 	return;
 
@@ -350,7 +351,7 @@ void afs_linux_vma_close(struct vm_area_struct *vmap)
 
 static int afs_linux_mmap(struct file *fp, struct vm_area_struct *vmap)
 {
-    struct vcache *vcp = (struct vcache*)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     cred_t *credp = crref();
     struct vrequest treq;
     int code;
@@ -451,7 +452,7 @@ static int afs_linux_release(struct inode *ip, struct file *fp)
 {
     int code = 0;
     cred_t *credp = crref();
-    struct vcache *vcp = (struct vcache*)ip;
+    struct vcache *vcp = ITOAFS(ip);
 
     AFS_GLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -486,7 +487,7 @@ static int afs_linux_fsync(struct file *fp, struct dentry *dp)
 #ifdef AFS_LINUX24_ENV
     lock_kernel();
 #endif
-    code = afs_fsync((struct vcache*)ip, credp);
+    code = afs_fsync(ITOAFS(ip), credp);
 #ifdef AFS_LINUX24_ENV
     unlock_kernel();
 #endif
@@ -510,7 +511,7 @@ int afs_linux_file_revalidate(kdev_t dev);
 static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
 {
     int code = 0;
-    struct vcache *vcp = (struct vcache*)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     cred_t *credp = crref();
 #ifdef AFS_LINUX24_ENV
     struct flock64 flock;
@@ -548,7 +549,7 @@ static int afs_linux_lock(struct file *fp, int cmd, struct file_lock *flp)
  */
 int afs_linux_flush(struct file *fp)
 {
-    struct vcache *vcp = (struct vcache *)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     int code = 0;
     cred_t *credp;
 
@@ -652,7 +653,7 @@ static int afs_linux_revalidate(struct dentry *dp)
     int code;
     cred_t *credp;
     struct vrequest treq;
-    struct vcache *vcp = (struct vcache*)dp->d_inode;
+    struct vcache *vcp = ITOAFS(dp->d_inode);
 
     AFS_GLOCK();
 #ifdef AFS_LINUX24_ENV
@@ -685,6 +686,91 @@ static int afs_linux_revalidate(struct dentry *dp)
     return -code ;
 }
 
+
+/* Validate a dentry. Return 1 if unchanged, 0 if VFS layer should re-evaluate.
+ * In kernels 2.2.10 and above, we are passed an additional flags var which
+ * may have either the LOOKUP_FOLLOW OR LOOKUP_DIRECTORY set in which case
+ * we are advised to follow the entry if it is a link or to make sure that 
+ * it is a directory. But since the kernel itself checks these possibilities
+ * later on, we shouldn't have to do it until later. Perhaps in the future..
+ */
+#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,10)
+static int afs_linux_dentry_revalidate(struct dentry *dp, int flags)
+#else
+static int afs_linux_dentry_revalidate(struct dentry *dp)
+#endif
+{
+    char *name;
+    cred_t *credp = crref();
+    struct vrequest treq;
+    struct vcache *lookupvcp = NULL;
+    int code, bad_dentry = 1;
+    struct sysname_info sysState;
+    struct vcache *vcp = ITOAFS(dp->d_inode);
+    struct vcache *parentvcp = ITOAFS(dp->d_parent->d_inode);
+
+    AFS_GLOCK();
+
+    sysState.allocked = 0;
+
+    /* If it's a negative dentry, then there's nothing to do. */
+    if (!vcp || !parentvcp)
+        goto done;
+
+    /* If it is the AFS root, then there's no chance it needs 
+       revalidating */
+    if (vcp == afs_globalVp) {
+	bad_dentry = 0;
+	goto done;
+    }
+
+    if (code = afs_InitReq(&treq, credp))
+        goto done;
+
+    Check_AtSys(parentvcp, dp->d_name.name, &sysState, &treq);
+    name = sysState.name;
+
+    /* First try looking up the DNLC */
+    if (lookupvcp = osi_dnlc_lookup(parentvcp, name, WRITE_LOCK)) {
+        /* Verify that the dentry does not point to an old inode */
+        if (vcp != lookupvcp)
+            goto done;
+        /* Check and correct mvid */
+        if (*name != '/' && vcp->mvstat == 2) 
+            check_bad_parent(dp);
+	vcache2inode(vcp);
+        bad_dentry = 0;
+        goto done;
+    }
+
+    /* A DNLC lookup failure cannot be trusted. Try a real lookup */
+    code = afs_lookup(parentvcp, name, &lookupvcp, credp);
+
+    /* Verify that the dentry does not point to an old inode */
+    if (vcp != lookupvcp)
+        goto done;
+
+    bad_dentry = 0;
+
+done:
+    /* Clean up */
+    if (lookupvcp)
+        afs_PutVCache(lookupvcp, WRITE_LOCK);
+    if (sysState.allocked)
+        osi_FreeLargeSpace(name);
+
+    AFS_GUNLOCK();
+    crfree(credp);
+
+    if (bad_dentry) {
+        shrink_dcache_parent(dp);
+        d_drop(dp);
+    }
+
+    return !bad_dentry;
+}
+
+#ifdef notdef
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,2,10)
 static int afs_linux_dentry_revalidate(struct dentry *dp, int flags)
 #else
@@ -694,10 +780,13 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
     int code;
     cred_t *credp;
     struct vrequest treq;
-    struct inode *ip = (struct inode *)dp->d_inode;
+    struct inode *ip = AFSTOI(dp->d_inode);
     
     unsigned long timeout = 3*HZ; /* 3 seconds */
     
+    if (!ip)
+	printk("negative dentry: %s\n", dp->d_name.name);
+ 
     if (!(flags & LOOKUP_CONTINUE)) {
 	long diff = CURRENT_TIME - dp->d_parent->d_inode->i_mtime;
 	
@@ -714,6 +803,7 @@ static int afs_linux_dentry_revalidate(struct dentry *dp)
  out_bad:
     return 0;
 }
+#endif
 
 /* afs_dentry_iput */
 static void afs_dentry_iput(struct dentry *dp, struct inode *ip)
@@ -764,7 +854,7 @@ int afs_linux_create(struct inode *dip, struct dentry *dp, int mode)
     vattr.va_mode = mode;
 
     AFS_GLOCK();
-    code = afs_create((struct vcache*)dip, name, &vattr, NONEXCL, mode,
+    code = afs_create(ITOAFS(dip), name, &vattr, NONEXCL, mode,
 		      (struct vcache**)&ip, credp);
 
     if (!code) {
@@ -813,10 +903,10 @@ int afs_linux_lookup(struct inode *dip, struct dentry *dp)
     struct vcache *vcp=NULL;
     const char *comp = dp->d_name.name;
     AFS_GLOCK();
-    code = afs_lookup((struct vcache *)dip, comp, &vcp, credp);
+    code = afs_lookup(ITOAFS(dip), comp, &vcp, credp);
 
     if (vcp) {
-	struct inode *ip = (struct inode*)vcp;
+	struct inode *ip = AFSTOI(vcp);
 	/* Reset ops if symlink or directory. */
 #if defined(AFS_LINUX24_ENV)
        if (S_ISREG(ip->i_mode)) {
@@ -831,17 +921,17 @@ int afs_linux_lookup(struct inode *dip, struct dentry *dp)
            ip->i_data.a_ops = &afs_symlink_aops;
            ip->i_mapping = &ip->i_data;
 	} else
-           printk("afs_linux_lookup: FIXME\n");
+           printk("afs_linux_lookup: ip->i_mode 0x%x  dp->d_name.name %s  code %d\n", ip->i_mode, dp->d_name.name, code);
 #else
 	if (S_ISDIR(ip->i_mode))
 	    ip->i_op = &afs_dir_iops;
 	else if (S_ISLNK(ip->i_mode))
 	    ip->i_op = &afs_symlink_iops;
 #endif
-    }
+    } 
     dp->d_time = jiffies;
     dp->d_op = afs_dops;
-    d_add(dp, (struct inode*)vcp);
+    d_add(dp, AFSTOI(vcp));
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -875,7 +965,7 @@ int afs_linux_link(struct dentry *olddp, struct inode *dip,
     d_drop(newdp);
 
     AFS_GLOCK();
-    code = afs_link((struct vcache*)oldip, (struct vcache*)dip, name, credp);
+    code = afs_link(ITOAFS(oldip), ITOAFS(dip), name, credp);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -889,27 +979,11 @@ int afs_linux_unlink(struct inode *dip, struct dentry *dp)
     const char *name = dp->d_name.name;
     int putback = 0;
 
-    if (!list_empty(&dp->d_hash)) {
-	d_drop(dp);
-	/* Install a definite non-existence if we're the only user. */
-#if defined(AFS_LINUX24_ENV)
-	if (atomic_read(&dp->d_count) == 1)
-#else
-	if (dp->d_count == 1)
-#endif
-	    putback = 1;
-    }
-
     AFS_GLOCK();
-    code = afs_remove((struct vcache*)dip, name, credp);
+    code = afs_remove(ITOAFS(dip), name, credp);
     AFS_GUNLOCK();
-    if (!code) {
-	d_delete(dp);
-	if (putback) {
-	    dp->d_time = jiffies;
-	    d_add(dp, NULL); /* means definitely does _not_ exist */
-    }
-    }
+    if (!code)
+	d_drop(dp);
     crfree(credp);
     return -code;
 }
@@ -930,7 +1004,7 @@ int afs_linux_symlink(struct inode *dip, struct dentry *dp,
 
     AFS_GLOCK();
     VATTR_NULL(&vattr);
-    code = afs_symlink((struct vcache*)dip, name, &vattr, target, credp);
+    code = afs_symlink(ITOAFS(dip), name, &vattr, target, credp);
     AFS_GUNLOCK();
     crfree(credp);
     return -code;
@@ -948,7 +1022,7 @@ int afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
     VATTR_NULL(&vattr);
     vattr.va_mask = ATTR_MODE;
     vattr.va_mode = mode;
-    code = afs_mkdir((struct vcache*)dip, name, &vattr, &tvcp, credp);
+    code = afs_mkdir(ITOAFS(dip), name, &vattr, &tvcp, credp);
 
     if (tvcp) {
 	tvcp->v.v_op = &afs_dir_iops;
@@ -957,8 +1031,9 @@ int afs_linux_mkdir(struct inode *dip, struct dentry *dp, int mode)
 #endif
 	dp->d_op = afs_dops;
 	dp->d_time = jiffies;
-	d_instantiate(dp, (struct inode*)tvcp);
+	d_instantiate(dp, AFSTOI(tvcp));
     }
+
     AFS_GUNLOCK();
     crfree(credp);
     return -code;
@@ -971,7 +1046,7 @@ int afs_linux_rmdir(struct inode *dip, struct dentry *dp)
     const char *name = dp->d_name.name;
 
     AFS_GLOCK();
-    code = afs_rmdir((struct vcache*)dip, name, credp);
+    code = afs_rmdir(ITOAFS(dip), name, credp);
 
     /* Linux likes to see ENOTEMPTY returned from an rmdir() syscall
      * that failed because a directory is not empty. So, we map
@@ -982,7 +1057,7 @@ int afs_linux_rmdir(struct inode *dip, struct dentry *dp)
     }
     
     if (!code) {
-	d_delete(dp);
+	d_drop(dp);
     }
 
     AFS_GUNLOCK();
@@ -1012,7 +1087,7 @@ int afs_linux_rename(struct inode *oldip, struct dentry *olddp,
 	d_drop(newdp);
     }
     AFS_GLOCK();
-    code = afs_rename((struct vcache*)oldip, oldname, (struct vcache*)newip,
+    code = afs_rename(ITOAFS(oldip), oldname, ITOAFS(newip),
 		      newname, credp);
     AFS_GUNLOCK();
 
@@ -1040,7 +1115,7 @@ static int afs_linux_ireadlink(struct inode *ip, char *target, int maxlen,
     struct iovec iov;
 
     setup_uio(&tuio, &iov, target, 0, maxlen, UIO_READ, seg);
-    code = afs_readlink((struct vcache*)ip, &tuio, credp);
+    code = afs_readlink(ITOAFS(ip), &tuio, credp);
     crfree(credp);
 
     if (!code)
@@ -1142,7 +1217,7 @@ int afs_linux_readpage(struct file *fp, struct page *pp)
 
     setup_uio(&tuio, &iovec, (char*)address, offset, PAGESIZE,
 	      UIO_READ, AFS_UIOSYS);
-    code = afs_rdwr((struct vcache*)ip, &tuio, UIO_READ, 0, credp);
+    code = afs_rdwr(ITOAFS(ip), &tuio, UIO_READ, 0, credp);
 #if LINUX_VERSION_CODE >= KERNEL_VERSION(2,4,0)
     unlock_kernel();
 #endif
@@ -1237,7 +1312,7 @@ int afs_linux_permission(struct inode *ip, int mode)
     if (mode & MAY_EXEC) tmp |= VEXEC;
     if (mode & MAY_READ) tmp |= VREAD;
     if (mode & MAY_WRITE) tmp |= VWRITE;
-    code = afs_access((struct vcache*)ip, tmp, credp);
+    code = afs_access(ITOAFS(ip), tmp, credp);
 
     AFS_GUNLOCK();
     crfree(credp);
@@ -1255,7 +1330,7 @@ int afs_linux_writepage_sync(struct inode *ip, struct page *pp,
                         unsigned long offset,
                         unsigned int count)
 {
-    struct vcache *vcp = (struct vcache *) ip;
+    struct vcache *vcp = ITOAFS(ip);
     char *buffer;
     loff_t base;
     int code = 0;
@@ -1308,7 +1383,7 @@ int afs_linux_updatepage(struct file *fp, struct page *pp,
 			 unsigned long offset,
 			 unsigned int count, int sync)
 {
-    struct vcache *vcp = (struct vcache *)FILE_INODE(fp);
+    struct vcache *vcp = ITOAFS(FILE_INODE(fp));
     u8 *page_addr = (u8*) afs_linux_page_address(pp);
     int code = 0;
     cred_t *credp;
