@@ -13,7 +13,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/SOLARIS/osi_vfsops.c,v 1.1.1.8 2001/07/14 22:19:51 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/SOLARIS/osi_vfsops.c,v 1.1.1.10 2002/05/10 23:44:13 hartmans Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -41,11 +41,14 @@ int afs_mount(struct vfs *afsp, struct vnode *amvp, struct mounta *uap,
 
     AFS_STATCNT(afs_mount);
 
-    if (!suser(credp))
+    if (!suser(credp)) {
+	AFS_GUNLOCK();
 	return EPERM;
+    }
     afsp->vfs_fstype = afsfstype;
 
     if (afs_globalVFS) { /* Don't allow remounts. */
+	AFS_GUNLOCK();
 	return EBUSY;
     }
 
@@ -68,8 +71,10 @@ int afs_unmount (struct vfs *afsp, struct AFS_UCRED *credp)
     AFS_GLOCK();
     AFS_STATCNT(afs_unmount);
 
-    if (!suser(credp))
+    if (!suser(credp)) {
+	AFS_GUNLOCK();
 	return EPERM;
+    }
     afs_globalVFS = 0;
     afs_shutdown();
 
@@ -117,13 +122,13 @@ int afs_root (struct vfs *afsp, struct vnode **avpp)
 	}
     }
     if (tvp) {
-	VN_HOLD((struct vnode *)tvp);
-	mutex_enter(&(((struct vnode*)tvp)->v_lock));
-	tvp->v.v_flag |= VROOT;
-	mutex_exit(&(((struct vnode*)tvp)->v_lock));
+	VN_HOLD(AFSTOV(tvp));
+	mutex_enter(&AFSTOV(tvp)->v_lock);
+	AFSTOV(tvp)->v_flag |= VROOT;
+	mutex_exit(&AFSTOV(tvp)->v_lock);
 
 	afs_globalVFS = afsp;
-	*avpp = (struct vnode *) tvp;
+	*avpp = AFSTOV(tvp);
     }
 
     afs_Trace2(afs_iclSetp, CM_TRACE_VFSROOT, ICL_TYPE_POINTER, *avpp,
@@ -241,6 +246,9 @@ void (*ufs_iupdatp)();
 int (*ufs_igetp)();
 void (*ufs_itimes_nolockp)();
 
+int (*afs_orig_ioctl)(), (*afs_orig_ioctl32)();
+int (*afs_orig_setgroups)(), (*afs_orig_setgroups32)();
+
 struct streamtab *udp_infop = 0;
 struct ill_s *ill_g_headp = 0;
 
@@ -259,15 +267,19 @@ extern Afs_syscall();
 
 afsinit(struct vfssw *vfsswp, int fstype)
 {
-    extern int afs_xioctl(), afs_xflock();
+    extern int afs_xioctl();
     extern int afs_xsetgroups();
 
     AFS_STATCNT(afsinit);
 
+    afs_orig_setgroups = sysent[SYS_setgroups].sy_callc;
+    afs_orig_ioctl = sysent[SYS_ioctl].sy_call;
     sysent[SYS_setgroups].sy_callc = afs_xsetgroups;
     sysent[SYS_ioctl].sy_call = afs_xioctl;
 
 #if defined(AFS_SUN57_64BIT_ENV)
+    afs_orig_setgroups32 = sysent32[SYS_setgroups].sy_callc;
+    afs_orig_ioctl32 = sysent32[SYS_ioctl].sy_call;
     sysent32[SYS_setgroups].sy_callc = afs_xsetgroups;
     sysent32[SYS_ioctl].sy_call = afs_xioctl;
 #endif
@@ -308,7 +320,7 @@ afsinit(struct vfssw *vfsswp, int fstype)
     if ( !nfs_checkauth ) afs_warn("nfs_checkauth not initialised");
 #endif
     ufs_iallocp = (int (*)()) modlookup("ufs", "ufs_ialloc");    
-    ufs_iupdatp = (int (*)()) modlookup("ufs", "ufs_iupdat");
+    ufs_iupdatp = (void (*)()) modlookup("ufs", "ufs_iupdat");
     ufs_igetp = (int (*)()) modlookup("ufs", "ufs_iget");    
     ufs_itimes_nolockp = (void (*)()) modlookup("ufs", "ufs_itimes_nolock");
     udp_infop = (struct streamtab *) modlookup("udp", "udpinfo");    
@@ -400,6 +412,9 @@ _init()
     extern struct bind *sb_hashtab[];
     struct modctl *mp = 0;
 
+    if (afs_sinited)
+	return EBUSY;
+
     if ((!(mp = mod_find_by_filename("fs", "ufs")) && 
 	!(mp = mod_find_by_filename(NULL, "/kernel/fs/ufs")) &&
 	!(mp = mod_find_by_filename(NULL, "sys/ufs"))) ||
@@ -481,8 +496,17 @@ _fini()
 {
     int code;
 
-    if (afs_sinited)
-	return (EBUSY);
+    if (afs_globalVFS)
+	return EBUSY;
+
+    if (afs_sinited) {
+	sysent[SYS_setgroups].sy_callc = afs_orig_setgroups;
+	sysent[SYS_ioctl].sy_call = afs_orig_ioctl;
+#if defined(AFS_SUN57_64BIT_ENV)
+	sysent32[SYS_setgroups].sy_callc = afs_orig_setgroups32;
+	sysent32[SYS_ioctl].sy_call = afs_orig_ioctl32;
+#endif
+    }
     code = mod_remove(&afs_modlinkage);
     return code;
 }
