@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_call.c,v 1.1.1.8 2001/09/11 14:24:37 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_call.c,v 1.1.1.12 2002/05/10 23:43:08 hartmans Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -191,6 +191,9 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	    afs_osi_Invisible();
 	    afs_RX_Running = 2;
 	    afs_osi_Wakeup(&afs_RX_Running);
+#ifndef UKERNEL
+	    afs_osi_RxkRegister();
+#endif
 	    rxk_Listener();
 	}
 #ifdef	AFS_SGI_ENV
@@ -326,6 +329,33 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	osi_FreeSmallSpace(tbuffer);
 	osi_FreeSmallSpace(tbuffer1);
     }
+    else if (parm == AFSOP_ADDCELLALIAS) {
+	/*
+	 * Call arguments:
+	 * parm2 is the alias name
+	 * parm3 is the real cell name
+	 */
+#if defined(AFS_SGI61_ENV) || defined(AFS_SUN57_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
+	size_t bufferSize;
+#else /* AFS_SGI61_ENV */
+	u_int bufferSize;	
+#endif /* AFS_SGI61_ENV */
+	char *aliasName = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
+	char *cellName = osi_AllocSmallSpace(AFS_SMALLOCSIZ);
+
+	AFS_COPYINSTR((char *)parm2, aliasName, AFS_SMALLOCSIZ, &bufferSize, code);
+	if (!code) AFS_COPYINSTR((char *)parm3, cellName, AFS_SMALLOCSIZ, &bufferSize, code);
+	if (!code) afs_NewCell(aliasName,	/* new entry name */
+			       0,		/* host list */
+			       CAlias,		/* flags */
+			       (char *) 0,	/* linked cell */
+			       0, 0,		/* fs & vl ports */
+			       0,		/* timeout */
+			       cellName);	/* real cell name */
+
+	osi_FreeSmallSpace(aliasName);
+	osi_FreeSmallSpace(cellName);
+    }
     else if (parm == AFSOP_CACHEINIT) {
 	struct afs_cacheParams cparms;
 
@@ -609,13 +639,28 @@ long parm, parm2, parm3, parm4, parm5, parm6;
 	int cellLen = (sizeArg & 0xffff0000) >> 16;
 	afs_int32 *kmsg = afs_osi_Alloc(kmsgLen);
 	char *cellname = afs_osi_Alloc(cellLen);
+
+#ifndef UKERNEL
+	afs_osi_MaskSignals();
+#endif
+	AFS_COPYIN((afs_int32 *)parm2, cellname, cellLen, code);
 	AFS_COPYIN((afs_int32 *)parm3, kmsg, kmsgLen, code);
-	if (!code) code = afs_AfsdbHandler(cellname, cellLen, kmsg);
+	if (!code) {
+	    code = afs_AfsdbHandler(cellname, cellLen, kmsg);
+	    if (*cellname == 1) *cellname = 0;
+	    if (code == -2) {	/* Shutting down? */
+		*cellname = 1;
+		code = 0;
+	    }
+	}
 	if (!code) AFS_COPYOUT(cellname, (char *)parm2, cellLen, code);
 	afs_osi_Free(kmsg, kmsgLen);
 	afs_osi_Free(cellname, cellLen);
     }
 #endif
+    else if (parm == AFSOP_SET_DYNROOT) {
+	code = afs_SetDynrootEnable(parm2);
+    }
     else
       code = EINVAL;
 
@@ -935,6 +980,10 @@ asmlinkage int afs_syscall(long syscall, long parm1, long parm2, long parm3,
     long linux_ret=0;
     long *retval = &linux_ret;
     long eparm[4]; /* matches AFSCALL_ICL in fstrace.c */
+#ifdef AFS_SPARC64_LINUX24_ENV
+    afs_int32 eparm32[4];
+#endif
+    /* eparm is also used by AFSCALL_CALL in afsd.c */
 #else
 #if defined(UKERNEL)
 Afs_syscall ()
@@ -995,7 +1044,26 @@ Afs_syscall ()
     uap->parm1 = parm1;
     uap->parm2 = parm2;
     uap->parm3 = parm3;
-    if (syscall == AFSCALL_ICL) {
+    if (syscall == AFSCALL_ICL || syscall == AFSCALL_CALL) {
+#ifdef AFS_SPARC64_LINUX24_ENV
+/* from arch/sparc64/kernel/sys_sparc32.c */
+#define AA(__x)                                \
+({     unsigned long __ret;            \
+       __asm__ ("srl   %0, 0, %0"      \
+                : "=r" (__ret)         \
+                : "0" (__x));          \
+       __ret;                          \
+})
+
+
+	if (current->thread.flags & SPARC_FLAG_32BIT) {
+	AFS_COPYIN((char*)parm4, (char*)eparm32, sizeof(eparm32), code);
+	eparm[0]=AA(eparm32[0]);
+	eparm[1]=AA(eparm32[1]);
+	eparm[2]=AA(eparm32[2]);
+#undef AA
+} else
+#endif
 	AFS_COPYIN((char*)parm4, (char*)eparm, sizeof(eparm), code);
 	uap->parm4 = eparm[0];
 	uap->parm5 = eparm[1];
@@ -1026,17 +1094,12 @@ Afs_syscall ()
 #endif
     } else if (uap->syscall == AFSCALL_SETPAG) {
 #ifdef	AFS_SUN5_ENV
-	struct cred *cred;
 	register proc_t *procp;
 
 	procp = ttoproc(curthread);
-	mutex_enter(&procp->p_crlock);
-	cred = procp->p_cred; 
 	AFS_GLOCK();
-	code =  afs_setpag(&cred);
+	code =  afs_setpag(&procp->p_cred);
 	AFS_GUNLOCK();
-	procp->p_cred = cred;
-	mutex_exit(&procp->p_crlock);
 #else
 	AFS_GLOCK();
 #if	defined(AFS_OSF_ENV) || defined(AFS_DARWIN_ENV) || defined(AFS_FBSD_ENV)
@@ -1201,6 +1264,12 @@ afs_shutdown()
 	afs_osi_Wakeup((char*)&afs_CacheTruncateDaemon);
 	afs_osi_Sleep(&afs_termState);
     }
+#ifdef AFS_AFSDB_ENV
+    afs_warn("AFSDB... ");
+    afs_StopAfsdb();
+    while (afs_termState == AFSOP_STOP_AFSDB)
+	afs_osi_Sleep(&afs_termState);
+#endif
 #if	defined(AFS_SUN5_ENV) || defined(RXK_LISTENER_ENV)
     afs_warn("RxEvent... ");
     /* cancel rx event deamon */
@@ -1208,6 +1277,9 @@ afs_shutdown()
 	afs_osi_Sleep(&afs_termState);
 #if defined(RXK_LISTENER_ENV)
     afs_warn("RxListener... ");
+#ifndef UKERNEL
+    afs_osi_UnmaskRxkSignals();
+#endif
     /* cancel rx listener */
     osi_StopListener(); /* This closes rx_socket. */
     while (afs_termState == AFSOP_STOP_RXK_LISTENER) 
