@@ -16,7 +16,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_callback.c,v 1.1.1.6 2001/09/20 06:07:11 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_callback.c,v 1.1.1.8 2002/05/10 23:43:09 hartmans Exp $");
 
 #include "../afs/sysincludes.h" /*Standard vendor system headers*/
 #include "../afs/afsincludes.h" /*AFS-based standard headers*/
@@ -157,7 +157,7 @@ int SRXAFSCB_GetCE(a_call, a_index, a_result)
     a_result->DataVersion = hgetlo(tvc->m.DataVersion);
     a_result->callback = afs_data_pointer_to_int32(tvc->callback);		/* XXXX Now a pointer; change it XXXX */
     a_result->cbExpires = tvc->cbExpires;
-    a_result->refCount = tvc->vrefCount;
+    a_result->refCount = VREFCOUNT(tvc);
     a_result->opens = tvc->opens;
     a_result->writers = tvc->execsOrWriters;
     a_result->mvstat = tvc->mvstat;
@@ -239,9 +239,15 @@ int SRXAFSCB_GetLock (a_call, a_index, a_result)
 	a_result->lock.exclLocked = ((struct afs_lock *)(tl->addr))->excl_locked;
 	a_result->lock.readersReading = ((struct afs_lock *)(tl->addr))->readers_reading;
 	a_result->lock.numWaiting = ((struct afs_lock *)(tl->addr))->num_waiting;
+#ifdef INSTRUMENT_LOCKS
 	a_result->lock.pid_last_reader = ((struct afs_lock *)(tl->addr))->pid_last_reader;
 	a_result->lock.pid_writer = ((struct afs_lock *)(tl->addr))->pid_writer;
 	a_result->lock.src_indicator = ((struct afs_lock *)(tl->addr))->src_indicator;
+#else
+	a_result->lock.pid_last_reader = 0;
+	a_result->lock.pid_writer = 0;
+	a_result->lock.src_indicator = 0;
+#endif
 	code = 0;
     }
 
@@ -1135,23 +1141,23 @@ int SRXAFSCB_GetCellServDB(
 
     /* search the list for the cell with this index */
     ObtainReadLock(&afs_xcell);
-    for (i=0, cq = CellLRU.next; cq != &CellLRU && i<= a_index; cq = tq, i++) {
-        tq = QNext(cq);
-	if (i == a_index) {
-	    tcell = QTOC(cq);
-	    p_name = tcell->cellName;
-	    for (j = 0 ; j < AFSMAXCELLHOSTS && tcell->cellHosts[j] ; j++) {
-		a_hosts[j] = ntohl(tcell->cellHosts[j]->addr->sa_ip);
-	    }
-	}
+
+    tcell = afs_GetCellByIndex(a_index, READ_LOCK, 0);
+
+    if (!tcell) {
+      i = 0;
+    } else {
+      p_name = tcell->cellName;
+      for (j = 0 ; j < AFSMAXCELLHOSTS && tcell->cellHosts[j] ; j++) {
+	a_hosts[j] = ntohl(tcell->cellHosts[j]->addr->sa_ip);
+      }
+      i = strlen(p_name);
+      afs_PutCell(tcell, READ_LOCK);
     }
 
-    if (p_name)
-	i = strlen(p_name);
-    else
-	i = 0;
     t_name = (char *)rxi_Alloc(i+1);
     if (t_name == NULL) {
+        ReleaseReadLock(&afs_xcell);
 #ifdef RX_ENABLE_LOCKS
 	AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
@@ -1211,6 +1217,7 @@ int SRXAFSCB_GetLocalCell(
      * the primary cell is when no other cell is explicitly marked as
      * the primary cell.  */
     ObtainReadLock(&afs_xcell);
+
     for (cq = CellLRU.next; cq != &CellLRU; cq = tq) {
         tq = QNext(cq);
 	tcell = QTOC(cq);
@@ -1229,6 +1236,7 @@ int SRXAFSCB_GetLocalCell(
 	plen = 0;
     t_name = (char *)rxi_Alloc(plen+1);
     if (t_name == NULL) {
+        ReleaseReadLock(&afs_xcell);
 #ifdef RX_ENABLE_LOCKS
 	AFS_GUNLOCK();
 #endif /* RX_ENABLE_LOCKS */
