@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_daemons.c,v 1.1.1.6 2001/09/11 14:24:39 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_daemons.c,v 1.1.1.8 2002/05/10 23:43:12 hartmans Exp $");
 
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
 #include "../afs/afsincludes.h"	/* Afs-based standard headers */
@@ -268,6 +268,7 @@ void afs_Daemon() {
 afs_CheckRootVolume () {
     char rootVolName[32];
     register struct volume *tvp;
+    int usingDynroot = afs_GetDynrootEnable();
 
     AFS_STATCNT(afs_CheckRootVolume);
     if (*afs_rootVolumeName == 0) {
@@ -276,7 +277,12 @@ afs_CheckRootVolume () {
     else {
 	strcpy(rootVolName, afs_rootVolumeName);
     }
-    tvp = afs_GetVolumeByName(rootVolName, LOCALCELL, 1, (struct vrequest *) 0, READ_LOCK);
+    if (usingDynroot) {
+	afs_GetDynrootFid(&afs_rootFid);
+	tvp = afs_GetVolume(&afs_rootFid, (struct vrequest *) 0, READ_LOCK);
+    } else {
+	tvp = afs_GetVolumeByName(rootVolName, LOCALCELL, 1, (struct vrequest *) 0, READ_LOCK);
+    }
     if (!tvp) {
 	char buf[128];
 	int len = strlen(rootVolName);
@@ -288,23 +294,25 @@ afs_CheckRootVolume () {
 	}
     }
     if (tvp) {
-	int volid = (tvp->roVol? tvp->roVol : tvp->volume);
-	afs_rootFid.Cell = LOCALCELL;
-	if (afs_rootFid.Fid.Volume && afs_rootFid.Fid.Volume != volid
-	    && afs_globalVp) {
-	    /* If we had a root fid before and it changed location we reset
-	     * the afs_globalVp so that it will be reevaluated.
-	     * Just decrement the reference count. This only occurs during
-	     * initial cell setup and can panic the machine if we set the
-	     * count to zero and fs checkv is executed when the current
-	     * directory is /afs.
-	     */
-	    AFS_FAST_RELE(afs_globalVp);
-	    afs_globalVp = 0;
+	if (!usingDynroot) {
+	    int volid = (tvp->roVol? tvp->roVol : tvp->volume);
+	    afs_rootFid.Cell = LOCALCELL;
+	    if (afs_rootFid.Fid.Volume && afs_rootFid.Fid.Volume != volid
+		&& afs_globalVp) {
+		/* If we had a root fid before and it changed location we reset
+		 * the afs_globalVp so that it will be reevaluated.
+		 * Just decrement the reference count. This only occurs during
+		 * initial cell setup and can panic the machine if we set the
+		 * count to zero and fs checkv is executed when the current
+		 * directory is /afs.
+		 */
+		AFS_FAST_RELE(afs_globalVp);
+		afs_globalVp = 0;
+	    }
+	    afs_rootFid.Fid.Volume = volid;
+	    afs_rootFid.Fid.Vnode = 1;
+	    afs_rootFid.Fid.Unique = 1;
 	}
-	afs_rootFid.Fid.Volume = volid;
-	afs_rootFid.Fid.Vnode = 1;
-	afs_rootFid.Fid.Unique = 1;
 	afs_initState = 300;    /* won */
 	afs_osi_Wakeup(&afs_initState);
 	afs_PutVolume(tvp, READ_LOCK);
@@ -330,7 +338,7 @@ afs_CheckRootVolume () {
 	    mp = (struct mount *) afs_globalVFS->vfs_data ;
 	    mp->m_rootgp = gget(mp, 0, 0, (char *)rootgp);
 	    afs_unlock(mp->m_rootgp);	/* unlock basic gnode */
-	    afs_vrele((struct vcache *) rootgp);  /* zap afs_root's vnode hold */
+	    afs_vrele(VTOAFS(rootgp));  /* zap afs_root's vnode hold */
 	}
     }
 #endif
@@ -381,9 +389,9 @@ void BPath(ab)
 	return;
     }
 #ifdef AFS_DEC_ENV
-    tvc = (struct vcache *) afs_gntovn(tvn);
+    tvc = VTOAFS(afs_gntovn(tvn));
 #else
-    tvc = (struct vcache *) tvn;
+    tvc = VTOAFS(tvn);
 #endif
     /* here we know its an afs vnode, so we can get the data for the chunk */
     tdc = afs_GetDCache(tvc, ab->parm[1], &treq, &offset, &len, 1);
@@ -532,7 +540,7 @@ struct brequest *afs_BQueue(aopcode, avc, dontwait, ause, acred, aparm0, aparm1,
 #ifdef	AFS_DEC_ENV
 		avc->vrefCount++;
 #else
-		VN_HOLD((struct vnode *)avc);
+		VN_HOLD(AFSTOV(avc));
 #endif
 	    }
 	    tb->refCount = ause+1;
@@ -770,7 +778,7 @@ afs_BioDaemon (nbiods)
 	    AFS_GLOCK();
 	    continue;
 	}
-	vcp = (struct vcache *)bp->b_vp;
+	vcp = VTOAFS(bp->b_vp);
 	if (bp->b_flags & B_PFSTORE) {	/* XXXX */
 	    ObtainWriteLock(&vcp->lock,404);	    
 	    if (vcp->v.v_gnode->gn_mwrcnt) {
@@ -1123,7 +1131,7 @@ afs_BioDaemon (nbiods)
 	    splx(s);
 	    continue;
 	}
-	vcp = (struct vcache *)bp->b_vp;
+	vcp = VTOAFS(bp->b_vp);
 	if (bp->b_flags & B_PFSTORE) {
 	    ObtainWriteLock(&vcp->lock,210);	    
 	    if (vcp->v.v_gnode->gn_mwrcnt) {
