@@ -18,7 +18,7 @@
 #include <afsconfig.h>
 #include "../afs/param.h"
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_volume.c,v 1.1.1.7 2001/09/20 06:07:16 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/afs/afs_volume.c,v 1.1.1.10 2002/05/10 23:43:27 hartmans Exp $");
 
 #include "../afs/stds.h"
 #include "../afs/sysincludes.h"	/* Standard vendor system headers */
@@ -72,6 +72,7 @@ afs_int32 fvTable[NFENTRIES];
 /* Forward declarations */
 static struct volume *afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 				   struct vrequest *areq, afs_int32 locktype);
+static struct volume *afs_NewDynrootVolume(struct VenusFid *fid);
 static inVolList();
 
 
@@ -224,6 +225,7 @@ void afs_ResetVolumes(struct server *srvp)
      for (vp=afs_volumes[j]; vp; vp=vp->next) {
 	for (k=0; k<MAXHOSTS; k++) {
 	   if (!srvp || (vp->serverHost[k] == srvp)) {
+	      vp->serverHost[k] = 0;
 	      afs_ResetVolumeInfo(vp);
 	      break;
 	   }
@@ -402,8 +404,12 @@ struct volume *afs_GetVolume(afid, areq, locktype)
 
     tv = afs_FindVolume(afid, locktype);
     if (!tv) {
-       bp = afs_cv2string(&tbuf[CVBS], afid->Fid.Volume);
-       tv = afs_NewVolumeByName(bp, afid->Cell, 0, areq, locktype);
+       if (afs_IsDynrootFid(afid)) {
+          tv = afs_NewDynrootVolume(afid);
+       } else {
+          bp = afs_cv2string(&tbuf[CVBS], afid->Fid.Volume);
+          tv = afs_NewVolumeByName(bp, afid->Cell, 0, areq, locktype);
+       }
     }
     return tv;
 } /*afs_GetVolume*/
@@ -549,6 +555,29 @@ struct volume *afs_GetVolumeByName(aname, acell, agood, areq, locktype)
   return(tv);
 }
 
+static struct volume *afs_NewDynrootVolume(struct VenusFid *fid) {
+    struct cell *tcell;
+    struct volume *tv;
+    struct vldbentry tve;
+    char *bp, tbuf[CVBS];
+
+    tcell = afs_GetCell(fid->Cell, READ_LOCK);
+    if (!tcell)
+	return (struct volume *) 0;
+    if (!(tcell->states & CHasVolRef))
+	tcell->states |= CHasVolRef;
+
+    bp = afs_cv2string(&tbuf[CVBS], fid->Fid.Volume);
+    memset(&tve, 0, sizeof(tve));
+    strcpy(tve.name, "local-dynroot");
+    tve.volumeId[ROVOL] = fid->Fid.Volume;
+    tve.flags = VLF_ROEXISTS;
+
+    tv = afs_SetupVolume(0, bp, &tve, tcell, 0, 0, 0);
+    afs_PutCell(tcell, READ_LOCK);
+    return tv;
+}
+
 int lastnvcode;
 static struct volume *afs_NewVolumeByName(char *aname, afs_int32 acell, int agood,
 				   struct vrequest *areq, afs_int32 locktype)
@@ -644,7 +673,7 @@ static struct volume *afs_NewVolumeByName(char *aname, afs_int32 acell, int agoo
 	} else
 	    code = -1;
     } while
-      (afs_Analyze(tconn, code, (struct VenusFid *) 0, areq,
+      (afs_Analyze(tconn, code, (struct VenusFid *) 0, &treq,
 		   -1, /* no op code for this */
 		   SHARED_LOCK, tcell));
 
