@@ -18,7 +18,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/vfsck/pass1.c,v 1.1.1.3 2001/07/14 22:24:41 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/vfsck/pass1.c,v 1.1.1.5 2002/05/11 00:03:19 hartmans Exp $");
 
 #define VICE
 #include <sys/param.h>
@@ -70,7 +70,8 @@ extern int ge_danger;
 static daddr_t badblk;
 static daddr_t dupblk;
 int pass1check();
-
+static int oldreported;
+ 
 pass1()
 {
 	register int c, i, j;
@@ -96,7 +97,7 @@ pass1()
 	/*
 	 * Find all allocated blocks.
 	 */
-	bzero((char *)&idesc, sizeof(struct inodesc));
+	memset((char *)&idesc, 0, sizeof(struct inodesc));
 	idesc.id_type = ADDR;
 	idesc.id_func = pass1check;
 	inumber = 0;
@@ -110,10 +111,9 @@ pass1()
 				continue;
 			dp = ginode(inumber);
 			if ((dp->di_mode & IFMT) == 0) {
-				if (bcmp((char *)dp->di_db, (char *)zino.di_db,
+				if (memcmp((char *)dp->di_db, (char *)zino.di_db,
 					NDADDR * sizeof(daddr_t)) ||
-				    bcmp((char *)dp->di_ib, (char *)zino.di_ib,
-					NIADDR * sizeof(daddr_t)) ||
+				    memcmp((char *)dp->di_ib, (char *)zino.di_ib, NIADDR * sizeof(daddr_t)) ||
 #if defined(ACLS) && defined(AFS_HPUX_ENV)
 				    dp->di_mode || dp->di_size || dp->di_contin) {
 				            if (dp->di_contin != 0) 
@@ -307,10 +307,19 @@ ignore_direct_block_check:
 #if	defined(AFS_SUN56_ENV) 
 			if ( OLDVICEINODE )
 			{
-			    /* This looks like a sol 2.5 AFS inode */
-			    printf("This vicep partition seems to contain pre Sol2.6 AFS inodes\n");
-			    printf("You should run the AFS file conversion utility before installing Sol 2.6\n");
-			    exit(100); /* unique return code? */
+			    if (yflag) {
+				if (!oldreported) {
+				    printf("This vicep partition seems to contain pre Sol2.6 AFS inodes\n");
+				    printf("You should run the AFS file conversion utility before installing Sol 2.6\n");
+				    printf("Continuing anyway.\n");
+				    oldreported++;
+				}
+			    } else {
+				/* This looks like a sol 2.5 AFS inode */
+				printf("This vicep partition seems to contain pre Sol2.6 AFS inodes\n");
+				printf("You should run the AFS file conversion utility before installing Sol 2.6\n");
+				exit(100); /* unique return code? */
+			    }
 			}
 #endif
 			statemap[inumber] =
