@@ -10,7 +10,7 @@
 #include <afsconfig.h>
 #include <afs/param.h>
 
-RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/viced/host.c,v 1.1.1.6 2001/09/11 14:35:35 hartmans Exp $");
+RCSID("$Header: /afs/sipb/project/openafs/debian/cvs/openafs/src/viced/host.c,v 1.1.1.9 2002/05/11 00:03:28 hartmans Exp $");
 
 #include <stdio.h>
 #include <errno.h>
@@ -114,8 +114,10 @@ static void GetCEBlock()
     register int i;
 
     block = (struct CEBlock *)malloc(sizeof(struct CEBlock));
-    if (!block)
+    if (!block) {
+	ViceLog(0, ("Failed malloc in GetCEBlock\n"));
 	ShutDownAndCore(PANIC);
+    }
 
     for(i = 0; i < (CESPERBLOCK -1); i++) {
 	Lock_Init(&block->entry[i].lock);
@@ -137,8 +139,10 @@ static struct client *GetCE()
 
     if (CEFree == 0)
 	GetCEBlock();
-    if (CEFree == 0)
+    if (CEFree == 0) {
+	ViceLog(0, ("CEFree NULL in GetCE\n"));
 	ShutDownAndCore(PANIC);
+    }
 
     entry = CEFree;
     CEFree = entry->next;
@@ -193,8 +197,10 @@ static void GetHTBlock()
     static int index = 0;
 
     block = (struct HTBlock *)malloc(sizeof(struct HTBlock));
-    if (!block)
+    if (!block) {
+	ViceLog(0, ("Failed malloc in GetHTBlock\n"));
 	ShutDownAndCore(PANIC);
+    }
 
 #ifdef AFS_PTHREAD_ENV
     for(i=0; i < (h_HTSPERBLOCK); i++)
@@ -264,14 +270,20 @@ int h_Release_r(host)
     register struct host *host;
 {	
     
-    if (!((host)->holds[h_holdSlot()] &= ~h_holdbit()) ) {
+    if (!((host)->holds[h_holdSlot()] & ~h_holdbit()) ) {
 	if (! h_OtherHolds_r(host) ) {
+	    /* must avoid masking this until after h_OtherHolds_r runs
+	       but it should be run before h_TossStuff_r */
+	    (host)->holds[h_holdSlot()] &= ~h_holdbit();
 	    if ( (host->hostFlags & HOSTDELETED) || 
 		(host->hostFlags & CLIENTDELETED) ) {
 		h_TossStuff_r(host);
 	    }		
-	}
-    }
+	} else 
+	    (host)->holds[h_holdSlot()] &= ~h_holdbit();
+    } else 
+      (host)->holds[h_holdSlot()] &= ~h_holdbit();
+
     return 0;
 }
 
@@ -736,6 +748,10 @@ h_TossStuff_r(host)
 	    FreeCE(client);
 	} else cp = &client->next;
     }
+
+    /* We've just cleaned out all the deleted clients; clear the flag */
+    host->hostFlags &= ~CLIENTDELETED;
+
     if (host->hostFlags & HOSTDELETED) {
 	register struct h_hashChain **hp, *th;
 	register struct rx_connection *rxconn;
@@ -924,6 +940,7 @@ struct host *h_GetHost_r(tcon)
     afs_int32 haddr;
     afs_int32 hport;
     int i, j, count;
+    char hoststr[16], hoststr2[16];
 
     haddr = rxr_HostOf(tcon);
     hport = rxr_PortOf(tcon);
@@ -944,7 +961,8 @@ retry:
 		/* Another thread is doing initialization */
 		h_Unlock_r(host);
 		if ( !held) h_Release_r(host);
-		ViceLog(125, ("Host %x starting h_Lookup again\n", host));
+		ViceLog(125, ("Host %s:%d starting h_Lookup again\n",
+			     afs_inet_ntoa_r(host->host, hoststr), host->port));
 		goto retry;
 	}
 	host->hostFlags &= ~ALTADDR;
@@ -961,8 +979,8 @@ retry:
 		 * that we maintain some extra callback state information */
 		if (host->interface) {
 		    ViceLog(0,
-			    ("Host %x used to support WhoAreYou, deleting.\n",
-			    host));
+			    ("Host %s:%d used to support WhoAreYou, deleting.\n",
+			    afs_inet_ntoa_r(host->host, hoststr), host->port));
 		    host->hostFlags |= HOSTDELETED;
 		    h_Unlock_r(host);
 		    if (!held) h_Release_r(host);
@@ -981,8 +999,8 @@ retry:
 		if ( !host->interface
 		  || !afs_uuid_equal(&interf.uuid, &host->interface->uuid) ) {
 		    ViceLog(25,
-			    ("Host %x has changed its identity, deleting.\n",
-			    host));
+			    ("Host %s:%d has changed its identity, deleting.\n",
+			    afs_inet_ntoa_r(host->host, hoststr), host->port));
 		    host->hostFlags |= HOSTDELETED;
 		    h_Unlock_r(host);
 		    if (!held) h_Release_r(host);
@@ -990,7 +1008,6 @@ retry:
 		    goto retry;
 		}
 	} else {
-	    char hoststr[16];
 	    afs_inet_ntoa_r(host->host, hoststr);
 	    ViceLog(0,("CB: WhoAreYou failed for %s:%d, error %d\n", 
 		       hoststr, ntohs(host->port), code));
@@ -1004,12 +1021,13 @@ retry:
 	if ( ! (host->hostFlags & ALTADDR) ) 
 	{
         	/* another thread is doing the initialisation */
-		ViceLog(125, ("Host %x waiting for host-init to complete\n",
-				host));
+		ViceLog(125, ("Host %s:%d waiting for host-init to complete\n",
+			     afs_inet_ntoa_r(host->host, hoststr), host->port));
 		h_Lock_r(host);
 		h_Unlock_r(host);
 		if ( !held) h_Release_r(host);
-		ViceLog(125, ("Host %x starting h_Lookup again\n", host));
+		ViceLog(125, ("Host %s:%d starting h_Lookup again\n",
+			     afs_inet_ntoa_r(host->host, hoststr), host->port));
 		goto retry;
 	}
 	/* We need to check whether the identity in the host structure
@@ -1024,8 +1042,8 @@ retry:
 		host->hostFlags |= HOSTDELETED;
 		h_Unlock_r(host);
 		if (!held) h_Release_r(host);
-		ViceLog(0,("CB: new identity for host %x, deleting\n",
-			   host->host));
+		ViceLog(0, ("CB: new identity for host %s:%d, deleting\n",
+			   afs_inet_ntoa_r(host->host, hoststr), host->port));
 		goto retry;
 	}
     } else {
@@ -1043,8 +1061,8 @@ retry:
 		    identP->valid = 0;
 		    rx_SetSpecific(tcon, rxcon_ident_key, identP);
 		    ViceLog(25,
-			    ("Host %x does not support WhoAreYou.\n",
-			    host->host));
+			    ("Host %s:%d does not support WhoAreYou.\n",
+			    afs_inet_ntoa_r(host->host, hoststr), host->port));
 		    code = 0;
 		} else if (code == 0) {
 		    interfValid = 1;
@@ -1052,7 +1070,8 @@ retry:
 		    identP->valid = 1;
 		    identP->uuid = interf.uuid;
 		    rx_SetSpecific(tcon, rxcon_ident_key, identP);
-		    ViceLog(25,("WhoAreYou success on %x\n", host->host));
+		    ViceLog(25, ("WhoAreYou success on %s:%d\n",
+				afs_inet_ntoa_r(host->host, hoststr), host->port));
 		}
 	    }
 	    if (code == 0 && !identP->valid) {
@@ -1068,8 +1087,9 @@ retry:
 		    if (!(held = h_Held_r(oldHost)))
 			h_Hold_r(oldHost);
 		    h_Lock_r(oldHost);
-		    ViceLog(25,("CB: new addr %x for old host %x\n",
-				host->host, oldHost->host));
+		    ViceLog(25, ("CB: new addr %s:%d for old host %s:%d\n",
+				afs_inet_ntoa_r(host->host, hoststr), host->port,
+				afs_inet_ntoa_r(oldHost->host, hoststr2), oldHost->port));
 		    host->hostFlags |= HOSTDELETED;
 		    h_Unlock_r(host);
 		    h_Release_r(host);
@@ -1083,15 +1103,14 @@ retry:
 						      &FS_HostUUID);
 		    H_LOCK
 		    if (code == 0) {
-			ViceLog(25,("InitCallBackState3 success on %x\n",
-				host->host));
+			ViceLog(25, ("InitCallBackState3 success on %s:%d\n",
+				    afs_inet_ntoa_r(host->host, hoststr), host->port));
 			assert(interfValid == 1);
 			initInterfaceAddr_r(host, &interf);
 		    }
 		}
 	   }
 	   if (code) {
-               char hoststr[16];
                afs_inet_ntoa_r(host->host, hoststr);
                ViceLog(0,("CB: RCallBackConnectBack failed for %s:%d\n", 
 			  hoststr, ntohs(host->port)));
@@ -1374,8 +1393,11 @@ ticket name length != 64
 	  code = pr_GetCPS(viceid, &client->CPS);
 	  H_LOCK
 	  if (code) {
-	    ViceLog(0, ("pr_GetCPS failed(%d) for user %d, host %x.%d\n",
-		    code, viceid, client->host->host, ntohs(client->host->port)));
+	    char hoststr[16];
+	    ViceLog(0, ("pr_GetCPS failed(%d) for user %d, host %s:%d\n",
+		       code, viceid,
+		       afs_inet_ntoa_r(client->host->host, hoststr),
+		       client->host->port));
 
 	    /* Although ubik_Call (called by pr_GetCPS) traverses thru
 	     * all protection servers and reevaluates things if no
@@ -1448,8 +1470,11 @@ int GetClient(tcon, cp)
     assert(client && client->tcon && rxr_CidOf(client->tcon) == client->sid);
     if (client &&
 	client->LastCall > client->expTime && client->expTime) {
-	ViceLog(1, ("Token for %s at %x.%d expired %d\n",
-		h_UserName(client), client->host->host, client->host->port, client->expTime));
+	char hoststr[16];
+	ViceLog(1, ("Token for %s at %s:%d expired %d\n",
+		h_UserName(client),
+		afs_inet_ntoa_r(client->host->host, hoststr),
+		client->host->port, client->expTime));
 	H_UNLOCK
 	return VICETOKENDEAD;
     }
@@ -1506,14 +1531,16 @@ static int h_PrintClient(host, held, file)
     int i;
     char tmpStr[256];
     char tbuffer[32];
+    char hoststr[16];
 
     H_LOCK
     if (host->hostFlags & HOSTDELETED) {
 	H_UNLOCK
 	return held;
     }
-    sprintf(tmpStr,"Host %x.%d down = %d, LastCall %s", host->host,
-	    host->port, (host->hostFlags & VENUSDOWN),
+    sprintf(tmpStr,"Host %s:%d down = %d, LastCall %s",
+	    afs_inet_ntoa_r(host->host, hoststr), host->port,
+	    (host->hostFlags & VENUSDOWN),
 	    afs_ctime((time_t *)&host->LastCall, tbuffer, sizeof(tbuffer)));
     STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
     for (client = host->FirstClient; client; client=client->next) {
@@ -1590,8 +1617,8 @@ static int h_DumpHost(host, held, file)
     char tmpStr[256];
 
     H_LOCK
-    sprintf(tmpStr, "ip:%x holds:%d port:%d hidx:%d cbid:%d lock:%x last:%u active:%u down:%d del:%d cons:%d cldel:%d\n\t hpfailed:%d hcpsCall:%u hcps [",
-	    host->host, host->holds, host->port, host->index, host->cblist,
+    sprintf(tmpStr, "ip:%x port:%d hidx:%d cbid:%d lock:%x last:%u active:%u down:%d del:%d cons:%d cldel:%d\n\t hpfailed:%d hcpsCall:%u hcps [",
+	    host->host, host->port, host->index, host->cblist,
 	    CheckLock(&host->lock), host->LastCall, host->ActiveCall, 
 	    (host->hostFlags & VENUSDOWN), host->hostFlags&HOSTDELETED, 
 	    host->Console, host->hostFlags & CLIENTDELETED, 
@@ -1609,8 +1636,16 @@ static int h_DumpHost(host, held, file)
 	    sprintf(tmpStr, " %x", host->interface->addr[i]);
 	    STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
 	}
-    sprintf(tmpStr, "]\n");
+    sprintf(tmpStr, "] holds: ");
     STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
+
+    for (i = 0 ; i < h_maxSlots ; i++) {
+      sprintf(tmpStr, "%04x", host->holds[i]);
+      STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
+    }
+    sprintf(tmpStr, " slot/bit: %d/%d\n", h_holdSlot(), h_holdbit());
+    STREAM_WRITE(tmpStr, strlen(tmpStr), 1, file);
+
     H_UNLOCK
     return held;
 
